options linesize=156;
/********* Documentation *******************************************************************************************************************;
County at DX Analysis VERSION 2.0

 Written by Information Management Services, Inc. September 2019.
 This program creates County at DX Analysis (data item #89) as defined by the data dictionary description for the version 18 NAACCR record
 layout, http://datadictionary.naaccr.org/?c=10
 Please read through the documentation below for instructions executing this program.

********************************************************************************************************************************************/
/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!     IMPORTANT - PLEASE READ      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!;

In order to execute this program you will need to specify the location and name for the following files:

  Input file references:
   Registry Incidence Data File, Logical Record Length (LRECL) 4048 Bytes (regfile reference below)
   Geocode Valid Cnty (geocode_valid_cnty.fmt, formats referenced below)

  Output file refernces:
   Updated Registry Incidence Data File, Logical Record Length (LRECL) 4048 Bytes (outfile reference below)

********************************************************************************************************************************************/

filename formats "geocode_valid_cnty.fmt";
%include formats;

proc format;
  value itemf
  2    = '2.   Reported County,county at dx geocode and reported county equal'
  2.1  = '2.1  Reported County,county at dx geocode is blank or unknown'
  2.2  = '2.2  Reported County,county at dx geocode is invalid for state'
  3    = '3    County at DX Geocode,certainty not blank/unk, reported county blank/unk'
  4    = '4    County at DX Geocode,certainty in 1 or 6'
  5    = '5    Reported County,certainty in 5'
  6    = '6    Reported County,certainty blank/unk'
  6.1  = '6.1  County at DX Geocode,certainty blank/unk, reported county blank/unk'
  7    = '7    Reported County,certainty in 2, 3, 4'
  9    = '9    Canadian or other Addr at DX-State'
  10   = '10   Blank State or Diagnosis Year'
  10.1 = '10.1 Blank Reported and County at DX Geo';

* Registry Incidence Input *;
*****************************************************************************;
/*NAACCR Version 18.0 input registry file, with the following fields included
   Addr at DX - State         [item #80,   column   124]
   County at DX Reported      [item #90,   column   135]
   County at DX Geocode 1990  [item #94,   column   155]
   Census Tr Cert 1970/80/90  [item #364,  column   166]
   County at DX Geocode 2000  [item #95,   column   169]
   Census Tr Certainty 2000   [item #365,  column   179]
   County at DX Geocode 2010  [item #96,   column   182]
   Census Tr Certainty 2010   [item #367,  column   192]
   Date of Diagnosis          [item #390,  column   544]                    */

filename regfile "directory and filename";

*OUTPUT FILE;
**** Output file Name - NAACCR Version 18 registry incidence file;
filename outfile "output.dat";


data registry_data;
  infile regfile truncover missover lrecl=4048 pad end=last_record;
  input incidence_record	     $1 - 4048 @; label incidence_record = 'Incidence Record';
  input state                  $124 - 125  @; label state            = 'State';
  input county                 $135 - 137  @; label county           = 'Reported County';
  input cntyDX90               $155 - 157  @; label cntyDX90         = 'County at DX Geocode 1990';
  input censtr90               $166 - 166  @; label censtr90         = 'Census Tr Cert 1970/80/90';
  input cntyDX00               $169 - 171  @; label cntyDX00         = 'County at DX Geocode 2000';
  input censtr00               $179 - 179  @; label censtr00         = 'Census Tr Certainty 2000';
  input cntyDX10               $182 - 184  @; label cntyDX10         = 'County at DX Geocode 2010';
  input censtr10               $192 - 192  @; label censtr10         = 'Census Tr Certainty 2010';
  input diagyear               $544 - 547  @; label diagyear         = 'Year of Diagnosis';
  state = upcase(state);

  if state = '  ' or diagyear = '    ' then do;
    county_analysis = '999'; item = 10;
    end;                                                                                               /* item 10 */
  else if state in('AB','BC','MB','NB','NL','NS','NT','NU','ON','PE','QC','SK','YT') then do;
    county_analysis = '998'; item = 9;
    end;                                                                                               /* item 9 */
  else do;
         if diagyear < '2000' then do; cnty_rec = cntyDX90; certainty = censtr90; end;
    else if diagyear < '2010' then do; cnty_rec = cntyDX00; certainty = censtr00; end;
    else do; cnty_rec = cntyDX10; certainty = censtr10; end;

    if county = '   ' and cnty_rec = '   ' then do;
      county_analysis = '999'; item = 10.1;
      end;                                                                                             /* item 10.1 */
    else if county = cnty_rec then do;
      county_analysis = county; item = 2;
      end;                                                                                             /* item 2 */
    else if cnty_rec in('   ','999') then do;
      county_analysis = county; item = 2.1;
      end;                                                                                             /* item 2.1 */
    else if input(put(input(state,regf.),z2.) || cnty_rec, vdcnty.) ^= 1 then do;
      county_analysis = county; item = 2.2;
      end;                                                                                             /* item 2.2 */
    else if certainty not in(' ','9') and county in('   ', '999') then do;
      county_analysis = cnty_rec; item = 3;
      end;                                                                                             /* item 3 */
    else do;
      if certainty in('1','6') then do;
        county_analysis = cnty_rec; item = 4;
        end;                                                                                           /* item 4 */
      else if certainty in('5') then do;
        county_analysis = county; item = 5;
        end;                                                                                           /* item 5 */
      else if certainty in (' ','9') then do;
        if county in('   ','999') then do;
          county_analysis = cnty_rec; item = 6.1;
          end;                                                                                         /* item 6.1 */
        else do;
          county_analysis = county; item = 6;
          end;                                                                                         /* item 6 */
        end;
      else if certainty in('2','3','4') then do;
        county_analysis = county; item = 7;
        end;                                                                                           /* item 7 */
      end;
  end;
run;

proc freq data=registry_data;
  table item / list missing;
  title "List of County at DX Analysis Bullets based on Description";
  format item itemf.;
run;

proc freq data=registry_data;
  where item = 10;
  table state*diagyear*county*county_analysis / list missing;
  title "Addr at DX-State (Data Item #80) or Reported County (Data Item #90) are Blank, County of DX Analysis set to 999";
run;
proc freq data=registry_data;
  where item = 10.1;
  table county*county_analysis*diagyear*certainty / list missing;
  title "Reported County (Data Item #90) and County Recode (intermediate value determined by diagyear, county at dx geocode and certainty) are Blank, County of DX Analysis set to 999";
run;
proc freq data=registry_data;
  where item = 2.2;
  table county*county_analysis*cnty_rec*diagyear*certainty / list missing;
  title "Invalid County Recode (intermediate value determined by diagyear, county at dx geocode and certainty)";
run;
proc freq data=registry_data;
  where item = 6.1;
  table diagyear*certainty*county*county_analysis*cnty_rec / list missing;
  title "Certainty Blank or Unknown and Reported County (Data Item #90), County Recode Value Valid (intermediate value determined by diagyear, county at dx geocode and certainty)";
run;

/*Create NAACCR version 18 file
  county_analysis - @150 Data Item 89 County at DX Analysis
*/
data _null_;
  set registry_data;
  file outfile lrecl=4048 pad;
  put @1    incidence_record $char4048.
      @150  county_analysis     $char3.;
run;
